/**
 * CMI : Cluster Method Invocation
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: carol@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * --------------------------------------------------------------------------
 * $Id: DistributedObjectInfo.java 1336 2007-10-22 16:38:55Z loris $
 * --------------------------------------------------------------------------
 */

package org.ow2.carol.cmi.controller.server;

import java.io.Serializable;

import net.jcip.annotations.ThreadSafe;

import org.ow2.carol.cmi.lb.data.LBPolicyData;

/**
 * Represents the replicated data for a clustered object.
 * @author The new CMI team
 */
@ThreadSafe
public class DistributedObjectInfo implements Serializable, Cloneable {

    /**
     * Id for serializable class.
     */
    private static final long serialVersionUID = 7128398689206804315L;

    /**
     * Name of the interface that the object implements.
     */
    private final String clusterName;

    /**
     * Name of the distributed object.
     */
    private final String objectName;

    /**
     * Name of the interface that the object implements.
     */
    private final String itfName;

    /**
     * Name of the business interface that the object implements.
     */
    private final String businessName;

    /**
     * Data of the LB policy.
     */
    private volatile LBPolicyData lbPolicyData;

    /**
     * Minimal size for the pool of CMIReferenceable.
     */
    private volatile int minPoolSize;

    /**
     * Maximal size for the pool of CMIReferenceable.
     */
    private volatile int maxPoolSize;

    /**
     * True if this object is replicated for high-availability.
     */
    private final boolean replicated;

    /**
     * Constructs informations for a clustered object with the default strategy for the given LB policy.
     * @param clusterName a name of the interface that the object implements
     * @param objectName a name of object
     * @param itfName a name of the interface that the object implements
     * @param businessName a name of the business interface that the object implements
     * @param lbPolicyData data of the LB policy
     * @param minPoolSize a minimal size for the pool of CMIReferenceable
     * @param maxPoolSize a maximal size for the pool of CMIReferenceable
     * @param replicated true if this object is replicated for high-availability
     */
    public DistributedObjectInfo(
            final String clusterName, final String objectName, final String itfName,
            final String businessName, final LBPolicyData lbPolicyData, final int minPoolSize,
            final int maxPoolSize, final boolean replicated) {
        this.clusterName = clusterName;
        this.objectName = objectName;
        this.itfName = itfName;
        this.businessName = businessName;
        this.lbPolicyData = lbPolicyData;
        this.minPoolSize = minPoolSize;
        this.maxPoolSize = maxPoolSize;
        this.replicated = replicated;
    }

    /**
     * @return a name of the interface that the object implements
     */
    public String getClusterName() {
        return clusterName;
    }

    /**
     * @return Name of the interface that the object implements.
     */
    public String getItfName() {
        return itfName;
    }

    /**
     * @return the businessName
     */
    public String getBusinessName() {
        return businessName;
    }

    /**
     * @param lbPolicyData data of the associated LB policy
     */
    public void setLBPolicyData(final LBPolicyData lbPolicyData) {
        this.lbPolicyData = lbPolicyData;
    }

    /**
     * @return data of the associated LB policy
     */
    public LBPolicyData getLBPolicyData() {
        return lbPolicyData;
    }

    /**
     * @return the minPoolSize
     */
    public int getMinPoolSize() {
        return minPoolSize;
    }

    /**
     * @param minPoolSize the minPoolSize to set
     */
    public void setMinPoolSize(final int minPoolSize) {
        this.minPoolSize = minPoolSize;
    }

    /**
     * @return the maxPoolSize
     */
    public int getMaxPoolSize() {
        return maxPoolSize;
    }

    /**
     * @param maxPoolSize the maxPoolSize to set
     */
    public void setMaxPoolSize(final int maxPoolSize) {
        this.maxPoolSize = maxPoolSize;
    }

    /**
     * @return the objectName
     */
    public String getObjectName() {
        return objectName;
    }

    @Override
    public boolean equals(final Object object) {
        if(object == null && !(object instanceof DistributedObjectInfo)) {
            return false;
        }
        DistributedObjectInfo distributedObjectInfo = (DistributedObjectInfo) object;
        return clusterName.equals(distributedObjectInfo.clusterName)
        && objectName.equals(distributedObjectInfo.objectName)
        && itfName.equals(distributedObjectInfo.itfName)
        && lbPolicyData.equals(distributedObjectInfo.lbPolicyData)
        && minPoolSize == distributedObjectInfo.minPoolSize
        && maxPoolSize == distributedObjectInfo.maxPoolSize;
    }

    @Override
    public String toString() {
        return "[clusterName="+clusterName+",objectName="+objectName+",itfName="
        +itfName+",lbPolicyData="+lbPolicyData+",minPoolSize="+minPoolSize+",maxPoolSize="+maxPoolSize;
    }

    /**
     * Return true if this object is replicated for high-availability.
     * The module 'ha' is so required.
     * @return true if this object is replicated for high-availability
     */
    public boolean isReplicated() {
        return replicated;
    }

    @Override
    public DistributedObjectInfo clone() throws CloneNotSupportedException {
        DistributedObjectInfo distributedObjectInfo =
            (DistributedObjectInfo) super.clone();
        distributedObjectInfo.lbPolicyData = lbPolicyData.clone();
        return distributedObjectInfo;
    }

}
